@@ -1135,10 +1135,7 @@ pub fn push_down_index(node: &Index) -> Option<CircuitRc> {
                     })
                     .collect(),
             );
-            let shape_non_axis = filter_out_idx(&inner.info().shape, &HashSet::from([inner.axis]))
-                .into_iter()
-                .collect();
-            if index_non_axis.is_identity(&shape_non_axis) {
+            if index_non_axis.is_identity(&inner.info().shape) {
                 return None;
             }
             // todo: drop concat parts that get indexed out
