@@ -76,6 +76,7 @@ static GENERAL_FUNCTIONS_SPECS: GILLazyPy<HashMap<String, GeneralFunctionSpec>>
             gfspecs_wrap!(py, "rsqrt", 0),
             gfspecs_wrap!(py, "gelu", 0),
             gfspecs_wrap!(py, "relu", 0),
+            gfspecs_wrap!(py, "step", 0),
             gfspecs_wrap!(py, "reciprocal", 0),
             gfspecs_wrap!(py, "log_exp_p_1", 0),
             gfspecs_wrap!(py, "gaussian_pdf", 0),
@@ -322,6 +323,7 @@ impl CircuitNodeAutoName for GeneralFunction {
 impl GeneralFunction {
     #[cfg(feature = "real-pyo3")]
     #[new]
+    #[args(name = "\"GeneralFunction\".to_owned()")]
     fn new(
         nodes: Vec<CircuitRc>,
         spec: GeneralFunctionSpec,
