@@ -5,7 +5,7 @@ use crate::{
     cached_lambda,
     py_types::{make_bytes_py, PyShape, Tensor},
     rearrange_spec::{OpSize, RearrangeSpec},
-    tensor_util::{Shape, TensorAxisIndex, TensorIndex, TorchDeviceDtype, TorchDeviceDtypeOp},
+    tensor_util::{Shape, Slice, TensorAxisIndex, TensorIndex, TorchDeviceDtype, TorchDeviceDtypeOp},
     util::AsOp,
 };
 pub use computational_node::{
@@ -798,7 +798,7 @@ pub enum CircuitConstructionError {
     #[error("index rank too high: {index_rank} vs {node_rank}")]
     IndexRankTooHigh { index_rank: usize, node_rank: usize },
 
-    #[error("Index out of bounds, {index:?} {shape:?}")]
+    #[error("Index {axis:?} out of bounds, index: {index:?} shape: {shape:?}. NB: Rust circuit slices don't saturate like Python ones do.")]
     IndexOutOfBounds {
         index: TensorIndex,
         shape: Shape,
@@ -807,6 +807,9 @@ pub enum CircuitConstructionError {
         l: usize,
     },
 
+    #[error("Start comes after its stop in slice {s:?}.")]
+    SliceDisordered { s: Slice },
+
     #[error("Scatter shape wrong, index: {index_shape:?} child: {shape:?} {index:?}")]
     ScatterShapeWrong {
         index: TensorIndex,
@@ -856,6 +859,11 @@ pub enum TensorEvalError {
     NotExplicitlyComputable { circuit: CircuitRc },
     #[error("python error {py_err:?}")]
     PythonError { py_err: Rc<PyErr> }, // PyErr doesn't have .clone, so Rc-ing
+    #[error("incompatible dtype circ:{circ:?} passed:{passed:?}")]
+    DeviceDtypeError {
+        circ: TorchDeviceDtypeOp,
+        passed: TorchDeviceDtypeOp,
+    },
 }
 
 impl From<TensorEvalError> for PyErr {
@@ -882,7 +890,21 @@ pub fn make_children_zero<T: CircuitNode>(circuit: &T) -> T {
 }
 
 pub fn evaluate_fn(circ: &Circuit) -> Result<Tensor, TensorEvalError> {
-    let device_dtype = get_compatible_dtype(circ);
+    evaluate_fn_dtype_device(circ, Default::default())
+}
+
+pub fn evaluate_fn_dtype_device(
+    circ: &Circuit,
+    dtype_device: TorchDeviceDtypeOp,
+) -> Result<Tensor, TensorEvalError> {
+    let device_dtype = dtype_device
+        .clone()
+        .combine(circ.info().device_dtype.clone())
+        .map_err(|_err| TensorEvalError::DeviceDtypeError {
+            circ: circ.info().device_dtype.clone(),
+            passed: dtype_device.clone(),
+        })?
+        .unwrap_or_defaults();
     #[apply(cached_lambda)]
     #[key(circ.info().hash, HashBytes)]
     fn recurse(circ: Circuit) -> Result<Tensor, TensorEvalError> {
@@ -1305,8 +1327,9 @@ impl PyCircuitBase {
         circuit_rust_to_py(CircuitRc(self.0.clone()))
     }
 
-    fn evaluate(&self) -> Result<Tensor, TensorEvalError> {
-        evaluate_fn(self)
+    #[args(device_dtype = "Default::default()")]
+    fn evaluate(&self, device_dtype: TorchDeviceDtypeOp) -> Result<Tensor, TensorEvalError> {
+        evaluate_fn_dtype_device(self, device_dtype)
     }
 }
 
