@@ -1,6 +1,6 @@
 use crate::{
     hashmaps::{AHashSet as HashSet, FxHashMap as HashMap},
-    py_types::{scalar_to_tensor, tensor_scale, un_flat_concat},
+    py_types::{scalar_to_tensor, tensor_scale, un_flat_concat, PY_UTILS},
 };
 use std::fmt::{self, Display};
 
@@ -13,7 +13,7 @@ use crate::{
 };
 
 use super::{
-    circuit_optimizer::OptimizationSettings,
+    circuit_optimizer::{OptimizationContext, OptimizationSettings},
     circuit_utils::{hash_to_node, toposort_circuit},
     get_compatible_dtype,
     prelude::*,
@@ -96,13 +96,26 @@ impl Schedule {
 
     pub fn evaluate(&self, settings: OptimizationSettings) -> Tensor {
         let eval = || {
-            if settings.adjust_numerical_scale {
+            let result = if settings.adjust_numerical_scale {
                 evaluate_schedule_adjust_numerical_scale(self, settings)
                     [&self.output_circuit.as_ref().unwrap().info().hash]
                     .clone()
             } else {
                 evaluate_schedule(self)[&self.output_circuit.as_ref().unwrap().info().hash].clone()
+            };
+            if self.device_dtype.device != "cpu".to_owned() {
+                Python::with_gil(|py| {
+                    PY_UTILS
+                        .torch
+                        .getattr(py, "cuda")
+                        .unwrap()
+                        .getattr(py, "synchronize")
+                        .unwrap()
+                        .call0(py)
+                        .unwrap()
+                });
             }
+            result
         };
         timed!(eval(), 10, settings.verbose >= 1)
     }
@@ -295,8 +308,10 @@ pub fn evaluate_schedule_adjust_numerical_scale(
         })
     };
     let clamp = |tup: &(Tensor, f64)| -> (Tensor, f64) {
-        let scale = tensor_scale(&tup.0).unwrap();
-        if scale > settings.numerical_scale_max || scale < settings.numerical_scale_min {
+        let scale: f64 = tensor_scale(&tup.0).unwrap();
+        if (scale > settings.numerical_scale_max || scale < settings.numerical_scale_min)
+            && scale != 0.0
+        {
             mul(tup, 1.0 / scale)
         } else {
             tup.clone()
@@ -713,20 +728,20 @@ fn subset_order_to_full_order(
 
 pub fn circuit_to_schedule(
     circuit: CircuitRc,
-    settings: OptimizationSettings,
+    context: &mut OptimizationContext,
 ) -> Result<Schedule, SchedulingError> {
-    let max_elements_in_memory = settings.max_memory / get_compatible_dtype(&circuit).size();
-    let chunk_size = max_elements_in_memory / settings.scheduling_num_mem_chunks;
+    let max_elements_in_memory = context.cache.max_tensor_elements;
+    let chunk_size = max_elements_in_memory / context.settings.scheduling_num_mem_chunks;
     let dag = circuits_to_dag(&[&**circuit]);
 
     let order_result = {
         // currently dag simplifications increase achievable memory usage unfortunately
         // todo make better simplifications
-        if settings.scheduling_simplify {
+        if context.settings.scheduling_simplify {
             // operate on subset dag
             let (sub_dag, kept_nodes) =
                 simplify_dag_for_scheduling_and_kept_nodes(&dag, chunk_size);
-            if settings.verbose >= 2 {
+            if context.settings.verbose >= 2 {
                 println!(
                     "dag sizes: orig: {} pruned: {}",
                     dag.node_costs.len(),
@@ -735,17 +750,17 @@ pub fn circuit_to_schedule(
             }
             schedule_dag_strategy_ints(
                 &sub_dag,
-                settings.verbose,
+                context.settings.verbose,
                 max_elements_in_memory,
-                settings.scheduling_num_mem_chunks,
+                context.settings.scheduling_num_mem_chunks,
             )
             .map(|sub_order| subset_order_to_full_order(&sub_order, &kept_nodes, &dag))
         } else {
             schedule_dag_strategy_ints(
                 &dag,
-                settings.verbose,
+                context.settings.verbose,
                 max_elements_in_memory,
-                settings.scheduling_num_mem_chunks,
+                context.settings.scheduling_num_mem_chunks,
             )
         }
     };
@@ -799,6 +814,9 @@ pub fn scheduled_evaluate(
     circuit: CircuitRc,
     settings: OptimizationSettings,
 ) -> Result<Tensor, SchedulingError> {
-    let schedule = circuit_to_schedule(circuit.clone(), settings)?;
+    let schedule = circuit_to_schedule(
+        circuit.clone(),
+        &mut OptimizationContext::new_settings_circuit(settings, circuit.clone()),
+    )?;
     Ok(evaluate_schedule(&schedule)[&circuit.info().hash].clone())
 }
