@@ -175,7 +175,7 @@ fn rust_circuit(py: Python<'_>, m: &PyModule) -> PyResult<()> {
         m
     )?)?;
     m.add_function(wrap_pyfunction!(
-        circuit::algebraic_rewrite::deep_optimize_einsums_py,
+        circuit::deep_rewrite::deep_optimize_einsums_py,
         m
     )?)?;
 
