@@ -90,25 +90,26 @@ pub struct Slice {
 impl Slice {
     pub fn to_unsigned_loc(x: i64, l: usize) -> usize {
         if x < 0 {
-            l.saturating_sub((-x) as usize)
+            l.checked_sub((-x) as usize).unwrap()
         } else {
-            l.min(x as usize)
+            assert!(x <= l as i64);
+            x as usize
         }
     }
 
-    fn start_u(self, l: usize) -> usize {
+    pub fn start_u(self, l: usize) -> usize {
         self.start.map(|x| Self::to_unsigned_loc(x, l)).unwrap_or(0)
     }
 
-    fn stop_u(self, l: usize) -> usize {
+    pub fn stop_u(self, l: usize) -> usize {
         self.stop.map(|x| Self::to_unsigned_loc(x, l)).unwrap_or(l)
     }
 
-    fn size(self, l: usize) -> usize {
+    pub fn size(self, l: usize) -> usize {
         self.stop_u(l).saturating_sub(self.start_u(l))
     }
 
-    fn canonicalize(self, l: usize) -> Self {
+    pub fn canonicalize(self, l: usize) -> Self {
         Self {
             start: Some(self.start_u(l) as i64),
             stop: Some(self.stop_u(l) as i64),
@@ -130,7 +131,7 @@ impl Slice {
         }
     }
 
-    fn is_identity(self, l: usize) -> bool {
+    pub fn is_identity(self, l: usize) -> bool {
         self.start_u(l) == 0 && self.stop_u(l) == l
     }
 }
@@ -450,6 +451,15 @@ impl TensorIndex {
                 &TensorAxisIndex::Slice(Slice { start, stop }) => {
                     check(at, start, l, true)?;
                     check(at, stop, l, true)?;
+
+                    if start > stop {
+                        // start and stop have to be Some(x) to get here
+                        let start_u = start.unwrap();
+                        let stop_u = stop.unwrap();
+                        if stop_u < 0 && start_u > stop_u + (l as i64) {
+                            get_err(at)?;
+                        }
+                    }
                 }
                 _ => (),
             }
